<?php
//
// +----------------------------------------------------------------------+
// | GeoClass                                                             |
// +----------------------------------------------------------------------+
// | Copyright (c) 2003 multimediamotz, Stefan Motz                       |
// +----------------------------------------------------------------------+
// | License (LGPL)                                                       |
// | This library is free software; you can redistribute it and/or        |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// +----------------------------------------------------------------------+
// | This library is distributed in the hope that it will be useful,      |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU     |
// | Lesser General Public License for more details.                      |
// +----------------------------------------------------------------------+
// | You should have received a copy of the GNU Lesser General Public     |
// | License along with this library; if not, write to the Free Software  |
// | Foundation Inc., 59 Temple Place,Suite 330, Boston,MA 02111-1307 USA |
// +----------------------------------------------------------------------+
// | Authors:  Stefan Motz   <stefan@multimediamotz.de>                   |
// |           Arne Klempert <arne@klempert.de>                           |
// | Version:  0.2.1                                                      |
// | Homepage: http://geoclassphp.sourceforge.net                         |
// +----------------------------------------------------------------------+
//
// $Id: Map.php,v 1.1 2003/09/15 17:13:19 klempert Exp $

require_once 'helpers/map.php';
require_once 'helpers/e00.php';

/**
 * Geo_Map
 *
 * this class provides functions to generate images from
 * GIS data and GeoObjects. In the future these methods
 * should be implemented in PEAR::Image_GIS
 *
 * @access   public
 * @package  Geo
 */
class Geo_Map extends e00 {

    var $colors = array();

    var $latitudeMin;
    var $latitudeMax;
    var $longitudeMin;
    var $longitudeMax;
    var $objects = array();
    var $imageMap = array();
    var $radius = 4;

    /**
     * constructor
     *
     * @param   mixed  $x  image-width (int) or path to image (string)
     * @param   int    $y  image-height
     * @return  void
     */
    function Geo_Map($x=false,$y=false) {
        $this->e00($x,$y);
        $this->color['white']    = $this->color(255, 255, 255);
        $this->color['red']      = $this->color(255, 0, 0);
        $this->color['black']    = $this->color(0, 0, 0);
        $this->color['green']    = $this->color(178, 237, 90);
        $this->color['blue']     = $this->color(148, 208, 255);
        $this->color['grey']     = $this->color(192, 192, 192);
        $this->color['darkgrey'] = $this->color(124, 124, 124);
    }

    /**
     * Sets the range of the map from overgiven degree-values
     *
     * container for API compatibility with PEAR::Image_GIS
     *
     * @access  public
     * @param   float   $x1
     * @param   float   $x2
     * @param   float   $y1
     * @param   float   $y2
     * @return  void
     */
    function setRange($x1,$x2,$y1,$y2) {
        $this->set_range($x1, $x2, $y1, $y2);
    }

    /**
     * Sets the range of the map from overgiven GeoObjects
     *
     * @access  public
     * @param   array   &$geoObjects  Array of GeoObjects
     * @param   float   $border       degrees
     * @return  void
     * @see     setRange(),setRangeByGeoObject()
     */
    function setRangeByGeoObjects(&$geoObjects,$border=0.1) {
        foreach($geoObjects AS $geoObject) {
            if (!$this->longitudeMin || ($geoObject->longitude < $this->longitudeMin)) {
                $this->longitudeMin = $geoObject->longitude;
            }
            if (!$this->longitudeMax || ($geoObject->longitude > $this->longitudeMax)) {
                $this->longitudeMax = $geoObject->longitude;
            }
            if (!$this->latitudeMin || ($geoObject->latitude < $this->latitudeMin)) {
                $this->latitudeMin = $geoObject->latitude;
            }
            if (!$this->latitudeMax || ($geoObject->latitude > $this->latitudeMax)) {
                $this->latitudeMax = $geoObject->latitude;
            }
        }

        $this->setRange(
            $this->longitudeMin - $border,
            $this->longitudeMax + $border,
            $this->latitudeMin - $border,
            $this->latitudeMax + $border
        );
    }

    /**
     * Sets the range of the map from an overgiven GeoObject
     *
     * @access  public
     * @param   array   &$geoObject  GeoObject
     * @param   float   $border      degrees
     * @return  void
     * @see     setRange(),setRangeByGeoObjects()
     */
    function setRangeByGeoObject(&$geoObject,$border=0.1) {
        if (!$this->longitudeMin || ($geoObject->longitude < $this->longitudeMin)) {
            $this->longitudeMin = $geoObject->longitude;
        }
        if (!$this->longitudeMax || ($geoObject->longitude > $this->longitudeMax)) {
            $this->longitudeMax = $geoObject->longitude;
        }
        if (!$this->latitudeMin || ($geoObject->latitude < $this->latitudeMin)) {
            $this->latitudeMin = $geoObject->latitude;
        }
        if (!$this->latitudeMax || ($geoObject->latitude > $this->latitudeMax)) {
            $this->latitudeMax = $geoObject->latitude;
        }

        $this->setRange(
            $this->longitudeMin - $border,
            $this->longitudeMax + $border,
            $this->latitudeMin - $border,
            $this->latitudeMax + $border
        );
    }

    /**
     * Adds a GeoObject to the map
     *
     * @access  public
     * @param   array   &$geoObject  GeoObject
     * @param   string  $color
     * @param   int     $radius
     * @return  void
     * @see     addGeoObjects()
     */
    function addGeoObject(&$geoObject,$color='black',$radius=false) {

    	if (!$radius) {
    		$radius = $this->radius;
    	}

        if ($radius) {
            for($i=1;$i<=$radius;$i++) {
                ImageArc(
                    $this->img,
                    $this->scale($geoObject->longitude, 'x'),
                    $this->scale($geoObject->latitude, 'y'),
                    $i,
                    $i,
                    0,
                    360,
                    $this->color[$color]
                );
            }
        }

        $this->imageMap[] = array(
            "name"=> $geoObject->getInfo(),
            "x"   => $this->scale($geoObject->longitude, 'x'),
            "y"   => $this->scale($geoObject->latitude, 'y'),
            "r"   => $radius?$radius:$this->radius
        );

    }

    /**
     * Adds GeoObjects to the map
     *
     * @access  public
     * @param   array   &$geoObjects  Array of GeoObjects
     * @param   string  $color
     * @return  void
     * @see     addGeoObject()
     */
    function addGeoObjects(&$geoObjects,$color='black') {
        foreach($geoObjects AS $geoObject) {
            $this->addGeoObject($geoObject,$color);
        }
    }

    /**
     * Adds an e00-file to the image
     *
     * container for API compatibility with PEAR::Image_GIS
     *
     * @access  public
     * @param   string  $data  path to e00-file
     * @return  boolean
     * @see     map::draw()
     */
    function addDataFile($data,$color='black') {
        if (file_exists($data)) {
        	$this->draw($data, $this->color[$color]);
        	return true;
        } else {
        	return false;
        }
    }

    /**
     * Saves the image
     *
     * container for API compatibility with PEAR::Image_GIS
     *
     * @access  public
     * @param   string  $file
     * @return  void
     * @see     map::dump()
     */
    function saveImage($file) {
        $this->dump($file);
    }

    /**
     * Creates an image map (html)
     *
     * @access  public
     * @param   string  $name  name of the ImageMap
     * @return  string  html
     */
    function getImageMap($name="map") {
        $html = '<map name="'.$name.'">';
        foreach($this->imageMap AS $koord) {
            $html .= '<area shape="circle" coords="'.round($koord['x']).','.round($koord['y']).','.$koord['r'].'" href="#" alt="'.$koord['name'].'">';
        }
        $html.='</map>';
        return $html;
    }

}
?>